/* 
 *  Copyright (c) 2010, 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "folder_size.hpp"

FolderSize::FolderSize(const QString& dir)
{
    init();
    directory = dir;
}

FolderSize::FolderSize(const QList<QModelIndex>& indexes, QFileSystemModel* filesystem)
{
    init();
    directory = "";
    m_indexes = indexes;
    m_filesystem = filesystem;
}

FolderSize::FolderSize()
{
    init();
    directory = QDir::currentPath();
}


void FolderSize::init()
{
    m_filesystem = NULL;
    m_totalFiles = 0;
    m_totalSize = 0;
    m_totalFolders = 0;
    m_running = false;
}


void FolderSize::run()
{
    m_running = true;
    m_totalFiles = 0;
    m_totalFolders = 0;
    
    if ( ! directory.isEmpty() )
    {
        QDirIterator it(directory, QDir::AllEntries | QDir::NoDotAndDotDot |
                                         QDir::Hidden, QDirIterator::Subdirectories);
                                         
        while (it.hasNext()) {
            if ( ! m_running )
                return;
            
            QFileInfo info(it.next());
            m_totalSize += info.size();
            m_totalFiles++;
        }
    }
    else if ( m_filesystem && m_indexes.size() > 0 )
    {
        foreach(QModelIndex index, m_indexes)
        {
            QFileInfo fileInfo = m_filesystem->fileInfo(index);
            m_totalSize += fileInfo.size();
            
            if ( fileInfo.isDir() )
                m_totalFolders++;
            else
                m_totalFiles++;
        }
    }

}

int FolderSize::getTotalFiles()
{
    return m_totalFiles;
}

qint64 FolderSize::getTotalSize()
{
    return m_totalSize;
}

void FolderSize::setDir(QString dir)
{
    directory = dir;
}

void FolderSize::stopProcessing()
{
    m_running = false;
}

