/* 
 *  Copyright (c) 2010, 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HPP_PROPERTIES
#define _HPP_PROPERTIES


#include "ui_properties.h"

#include <QtGui/QDialog>
#include <QtGui/QWidget>
#include <QtCore/QObject>
#include <QTextStream>
#include <QPushButton>
#include <QtDebug>
#include <QTimer>
#include <QFileSystemModel>
#include <QtGui/QDialog>
#include <QtGui/QWidget>
#include <QDateTime>
#include <QFile>
#include <QFlags>
#include <QCheckBox>
#include <QModelIndex>
#include <QString>
#include <QList>

#include "folder_size.hpp"
#include "scope_model.hpp"

//class QFlags<QFile::Permission>;

class PropertiesDialog : public QDialog, private Ui::PropertiesDialog
{
    Q_OBJECT
    
    public:

        PropertiesDialog(QList<QModelIndex>& index, ScopeModel& filesystem, QWidget * p_parent = 0);
        ~PropertiesDialog();
        
        inline bool isSettingChanged() const { return m_SettingChanged; }
        
        void setName(QString);
        void setSize(qint64);
        void setPath(QString);
        void setLastModified(QDateTime);
        void setLastRead(QDateTime);
        void setPermissions( QFlags<QFile::Permission> perms);
        void setPerm(QString);
        void applyChanges();

    private:

        class FileAtributes //to use later because of the metadata and other attributes
        {
            QString name;
            
            public:
                void setName(QString& n){ name = n; };
                QString getName();
        };

        FileAtributes file_atributes;
        QFileInfo m_fileInfo;
        QFileInfo m_filesInfo;
        QModelIndex m_index;
        QList<QModelIndex> m_indexes;
        QString m_path;
        ScopeModel *m_filesystem;
        FolderSize *fs;
        QTimer timer;
        bool m_SettingChanged;
        void makeConnections();
        void enablePermissions(bool enable);
        void setInfo();
        void getFolderSize();
        
    private slots:
        void setFolderSize();
        void nameEdited(const QString &);
    
};



#endif
