/* 
 *  Copyright (c) 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "file_icon_provider.hpp"
#include <QProcess>
#include <QtDebug>
 
QHash<QString, QIcon> FileIconProvider::m_iconsCache; 
QHash<QString, QIcon> FileIconProvider::m_thumbsCache;
QFreeDesktopMime FileIconProvider::m_mime; 


FileIconProvider::FileIconProvider()
{
    loadMimeTypes();
    m_showThumnails = false;
    m_imageExt << "jpg" << "jpeg" << "png" << "gif" << "svg" << "svgz";
}

QIcon FileIconProvider::icon(const QFileInfo & info )const
{
    /*QString cmd = QString("xdg-mime query filetype \"%1\"").arg(info.filePath());
    QProcess process;
    process.start(cmd);
    process.waitForStarted();
    process.waitForFinished();
    QString mime = process.readAll().trimmed();
    QString mimeEdited = mime;*/
    
    //QString iconType = m_mime.genericIconName(m_mime.fromFile(info.absoluteFilePath()));//m_mime.genericIconName();
    
    
    QString suffix = info.suffix();
    QString filePath = info.absoluteFilePath();
    QString mimeType, mimeIcon;
    QIcon icon;
    
    if ( info.isDir() )
        suffix = "";
    
    if ( m_showThumnails && m_thumbsCache.contains(filePath) )
        return m_thumbsCache[filePath];
    
    if ( ! suffix.isEmpty())
    {
        
        
        //special case for icon thumbs
        if ( m_showThumnails && m_imageExt.contains(suffix) )
        {
            QPixmap pixmap(filePath);
            if ( ! pixmap.isNull() )
            {
                pixmap = pixmap.scaled(m_iconSize.width(), m_iconSize.height(), Qt::KeepAspectRatioByExpanding);
                m_thumbsCache[filePath] = QIcon(pixmap);
                return m_thumbsCache[filePath];
            }
        }
        
        if ( m_iconsCache.contains(suffix) )
        {
            return m_iconsCache[suffix];
        }
        else if ( m_mimeIcons.contains(suffix) )
        {
            icon = _getIcon(m_mimeIcons[suffix]);
            if ( ! icon.isNull() )
            {
                m_iconsCache[suffix] = icon;
                return icon;
            }
        }
    }
        
    //mime = m_mime.fromFileName(info.fileName());
    //if ( mime.isEmpty() )
    //mimeType = m_mime.fromFile(info.absoluteFilePath());
    
        
    /*if ( m_iconsCache.contains(mimeType) )
        return m_iconsCache[mimeType];
        
    mimeIcon = mimeType;
    
    if ( mimeIcon.contains("/") )
        mimeIcon.replace("/", "-");*/
        
    //qDebug() << iconType << info.fileName();
        
    //qDebug() << "getting new icon..." << iconType;
    //QIcon icon = _getIcon(mimeIcon);
    
    /*if ( icon.isNull() && mimeEdited.contains("-x-") )
    {  
        mimeEdited = mimeEdited.split("-x-")[0] + "-x-generic"; 
        icon = _getIcon(mimeEdited);
    }*/
    
    /*if ( icon.isNull() )
    {
        qDebug() << "QFileIconProvider::icon(info)";
        icon = QFileIconProvider::icon(info);
    } 
    //if ( ! icon.isNull() )
    if ( ! suffix.isEmpty() )
        m_iconsCache.insert(suffix, icon);
    else
        m_iconsCache.insert(mimeType, icon);
    qDebug() << "Defaulting to QFileIconProvider";*/
    return QFileIconProvider::icon(info);
}


/*=========================== Group Class =============================*/

Group::Group(const QString& name)
{
    m_name = name;
    m_size = 0;
    m_minsize = m_size;
    m_maxsize = m_size;
    m_threshold = 2;
    m_type = "Scaled";
}

void Group::addValue(const QString& line)
{
    bool ok;
    
    if ( line.contains("size", Qt::CaseInsensitive) )
    {
         m_size = getValueFromLine(line).toInt(&ok);
    }
    else if ( line.contains("type", Qt::CaseInsensitive) )
        m_type = getValueFromLine(line);
    else if ( line.contains("minsize", Qt::CaseInsensitive) )
    {
         m_minsize = getValueFromLine(line).toInt(&ok);
         if ( ! ok ) m_minsize = m_size;
    }
    else if ( line.contains("maxsize", Qt::CaseInsensitive) )
    {
         m_maxsize = getValueFromLine(line).toInt(&ok);
         if ( ! ok ) m_maxsize = m_size;
    }
    else if ( line.contains("threshold", Qt::CaseInsensitive) )
    {
         m_threshold = getValueFromLine(line).toInt(&ok);
         if ( ! ok ) m_threshold = 2;
    }
}


/*=========================== IndexTheme Class =============================*/

IndexTheme::IndexTheme(const QString& path)
{
    m_test = 0;
    setPath(path);
}

IndexTheme::IndexTheme()
{
    m_test = 0;
    m_path = "";
}


void IndexTheme::setPath(const QString& path)
{
    bool addValues = false;
    Group group;
    
    m_path = path;
    
    foreach(QString line, getData(path))
    {
        line = line.trimmed();
        
        if ( line.startsWith("#") || line.isEmpty() )
            continue;
        
        if ( line.contains("inherits", Qt::CaseInsensitive) )
        {
            if ( line.contains(",") )
                m_inherits = getValueFromLine(line).split(",");
            else
                m_inherits << getValueFromLine(line);
            continue;
        }
           
        if ( addValues && line.contains("=") )
            group.addValue(line);
            
        if ( line.contains("/") && line.contains("[") && line.contains("]") )
        {
            if ( addValues )
                m_groups << group;
                
            group = Group(line.split("[")[1].split("]")[0]);
            addValues = true;
        }
    }
}

QStringList& IndexTheme::match(int iconSize)
{
    QStringList validGroups;
    
    if ( m_matchesCache.contains(iconSize) )
        return m_matchesCache[iconSize];
        
    foreach(Group group, m_groups)
    {
        if ( group.type() == "Fixed" )
        {
            if ( group.size() == iconSize )
                validGroups << group.name();
        }
        else if ( group.type() == "Scaled" )
        {
                if ( group.size() >= group.minsize() && group.size() <= group.maxsize() )
                    validGroups << group.name();
        }
        else if ( group.type() ==  "Threshold" )
        {
            if ( group.size() >= group.size() - group.threshold() && group.size() <= group.size() + group.threshold() )
                validGroups << group.name();
        }
    }
    
    m_matchesCache[iconSize] = validGroups;
    return m_matchesCache[iconSize];
    
}

bool IndexTheme::operator==(IndexTheme& other)
{ 
    return m_path == other.path(); 
}

bool IndexTheme::operator==(const QString& path)
{ 
    return m_path == path; 
}


/*=========================== Icon Class =============================*/

void FileIconProvider::loadMimeTypes()
{
    QFile mimeGlobs("/usr/share/mime/globs");
    QFile mimeGenericIcon("/usr/share/mime/generic-icons");
    
    if ( ! mimeGlobs.open(QIODevice::ReadOnly | QIODevice::Text) )
        return;
        
    QTextStream fileStream(&mimeGlobs);
    QString line, suffix, mimeIcon, mime;
    
    while( ! fileStream.atEnd() )
    {
        line = fileStream.readLine().trimmed();
        if ( ! line.contains(":") ) continue;
            
        suffix = line.split(":")[1];
        mime = line.split(":")[0];
        
        if ( suffix.contains("*.") )
            suffix.replace("*.", "");
        
        if ( mime.contains("/") )
            mime.replace("/", "-");
        
        m_mimeIcons[suffix] = mime;
    }
    
    mimeGlobs.close();
    
    /*if ( ! mimeGenericIcon.open(QIODevice::ReadOnly | QIODevice::Text) )
        return;
        
    fileStream.setDevice(&mimeGenericIcon);
 
    while( ! fileStream.atEnd() )
    {
        line = fileStream.readLine().trimmed();
        if ( ! line.contains(":") ) continue;
        
        mime = line.split(":")[0].replace("/", "-");
        
        foreach(QString suf, m_mimeIcons.keys())
            if ( m_mimeIcons[suf] == mime )
                m_mimeIcons[suf] = line.split(":")[1];
    }
    
    mimeGenericIcon.close();*/
}

QIcon FileIconProvider::_getIcon(const QString& name, int size) const
{
    
    #if defined(Q_WS_X11)

    #if QT_VERSION >= 0x040600
        return QIcon::fromTheme(name);
    #else

        if ( m_iconTheme.isEmpty() || m_iconTheme.isNull() )
        {
            QString desktop = getDesktopEnv();
            if ( desktop == "kde" )
                m_iconTheme = "oxygen";
            else if ( desktop == "gnome" || desktop == "xfce" || desktop == "lxde" )
                m_iconTheme = "gnome";
            else
                m_iconTheme = "hicolor";
        }
        if ( m_baseDirs.isEmpty() )
            m_baseDirs = FileIconProvider::getBaseDirectories();
        if ( m_filters.isEmpty() )
            m_filters << "*.png" << "*.xpm" << "*.svg";
        
        QString path = FileIconProvider::lookupIcon(name, m_iconTheme, size);

        if ( path.isEmpty() )
        {
            //QString parent = m_indexThemeData.inherits();//getValueFromData(m_indexThemeData, "Inherits");
            QStringList parents = m_indexTheme->inherits();
            if ( parents.isEmpty() && m_indexTheme->name() != "hicolor")
                parents << "hicolor";
            
            foreach(QString parent, parents)
            {
                path = lookupIcon(name, parent, size);
                if ( ! path.isEmpty() )
                    break;
            }
        }
        
        if ( path.isEmpty() )
           path = lookupFallbackIcon(name);

        if ( QFile::exists(path) )
            return QIcon(QPixmap(path));
            
        return QIcon();
    
    #endif
    
    #endif
    
    return QIcon();
} 

QString FileIconProvider::lookupFallbackIcon(const QString& iconName)
{
    
    foreach(QDir basedir, m_baseDirs)
    {
        basedir.setNameFilters(m_filters);
        foreach(QFileInfo fileInfo, basedir.entryInfoList(QDir::Files | QDir::NoSymLinks))
        {
            if ( fileInfo.baseName() == iconName )
                return fileInfo.absoluteFilePath(); 
        }
    }
    
    return "";
}

QString FileIconProvider::lookupIcon(const QString& iconName, const QString& theme, int size)
{
    QString iconPath("");
    QDir themeDir;
    QString iconsPath;
    QStringList pathParts;

    foreach(QDir basedir, m_baseDirs)
    {
        themeDir.setPath(basedir.path());
        if ( ! themeDir.cd(theme) ) continue;
        
        
        m_indexTheme = getIndexThemeData(themeDir.absoluteFilePath("index.theme"));
        pathParts.clear();
        
        if ( m_indexTheme->isEmpty() ) 
            pathParts << getPathParts(themeDir.path(), size);
        else
            pathParts = m_indexTheme->match(size);
        
        for(int i=pathParts.size()-1; i>= 0; --i)
        {
            
            iconsPath = themeDir.absoluteFilePath(pathParts[i]);
            if ( QFile::exists(iconsPath) )
            {
                iconPath = getIconPathInDir(iconName, iconsPath);
                if ( ! iconPath.isEmpty() )
                    return iconPath;
            }
            
        }
    }
    
    return iconPath;
    
}

QString FileIconProvider::getPathParts(const QString& p, int iconSize)
{
    int i;
    QString path = p;
    QDir baseDir(path);
    QHash<QString, QStringList> indexer;
    QStringList parts;
    QString basePath = path;
    QString size = QString::number(iconSize);
    
    while(1)
    {
        QDir dir(path);

        if ( indexer.contains(dir.path()) )
        {
            if ( indexer[dir.path()].isEmpty() )
            {
                if ( dir == baseDir )
                    break;
                else
                {
                    dir.cdUp();
                    path = dir.path();
                }
            }
            else
            {
                dir.cd(indexer[dir.path()].takeFirst());
                path = dir.path();
            }
        }
        
        if ( ! indexer.contains(dir.path()) )
        {
            indexer[dir.path()] = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
        
            if ( indexer[dir.path()].isEmpty() )
            {
                path.remove(basePath);
                if ( path.at(0) == '/' )
                    path.remove(0, 1);
                parts << path;
                path = dir.path();
            }
        }
    }
    
    foreach(QString part, parts)
    {
        if ( part.contains(size) )
            return part;
    }

    return parts.last(); //fallback
}

IndexTheme* FileIconProvider::getIndexThemeData(const QString& path)
{
    int i;
     
    for(i=0; i < m_indexThemeCache.size(); ++i)
        if ( (*m_indexThemeCache[i]) == path )
            break;
           
    if ( i == m_indexThemeCache.size() )
    {
        m_indexThemeCache << new IndexTheme(path); 
        return m_indexThemeCache.last();
    }
    
    return m_indexThemeCache[i];
}


QString FileIconProvider::getIconPathInDir(const QString& iconName, const QString& dirPath)
{
    QDir dir(dirPath);
    
    if ( ! dir.exists() )
        return "";
    
    if ( ! m_dirsCache.contains(dirPath) )
    {
        dir.setNameFilters(m_filters);
        m_dirsCache[dirPath] = dir.entryList(QDir::Files);
    }
    
    foreach(QString file, m_dirsCache[dirPath])
    {
        if ( file.contains(iconName) )
        {
            QFileInfo fileInfo(dir.absoluteFilePath(file));
            if ( fileInfo.baseName() == iconName )
                return fileInfo.absoluteFilePath();
        }
    }
    
    return "";
}

QList<QDir> FileIconProvider::getBaseDirectories()
{
    QList<QDir> baseDirs;
    
    QDir home = QDir::home();
    if ( home.cd(".icons") )
        baseDirs << home;
    
    QString xdgDataDirs(getenv("XDG_DATA_DIRS"));

    if ( xdgDataDirs.contains(":") )
    {
        QStringList dirs = xdgDataDirs.split(":");
#if QT_VERSION >= 0x040500
        dirs.removeDuplicates();
#else
        removeStringDuplicates(dirs);
#endif
        foreach(QString xdgDataDir, dirs)
        {
            QDir dir(xdgDataDir);
            if ( dir.cd("icons") )
                baseDirs << dir;
        }
           
    }
    else
    {
        QDir dir(xdgDataDirs);
            if ( dir.cd("icons") )
                baseDirs << dir;
    }
        
    
    baseDirs << QDir("/usr/share/pixmaps"); 
    
    return baseDirs;
}


void FileIconProvider::setIconSize(int width, int height)
{
    m_iconSize.scale(width, height, Qt::IgnoreAspectRatio);

    foreach(QString path, m_thumbsCache.keys())
    {
        QPixmap pixmap(path);
        pixmap.scaled(m_iconSize, Qt::KeepAspectRatioByExpanding);
        m_thumbsCache[path] = QIcon(pixmap);
    }
}

void FileIconProvider::setIconSize(QSize size)
{
    setIconSize(size.width(), size.height());
}

void FileIconProvider::showThumbnails(bool show)
{
    m_showThumnails = show;
}
