/* 
 *  Copyright (c) 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef FILE_ICON_PROVIDER
#define FILE_ICON_PROVIDER

#include <QIcon>
#include <QString>
#include <QList>
#include <QDir>
#include <QHash>
#include <QFileIconProvider>
#include <QTextStream>

#include "utils.h"
#include "freedesktopmime.hpp"

class Group
{
    int m_size;
    int m_minsize;
    int m_maxsize;
    int m_threshold;
    QString m_type;
    QString m_name;
    
    public:
        Group(const QString& name="");
        ~Group(){}
        QString& name() { return m_name; }
        int size() {return m_size; }
        void setSize(int size) { m_size = size; }
        int minsize() { return m_minsize; }
        void setMinsize(int minsize) { m_minsize = minsize; }
        int maxsize() { return m_maxsize; }
        void setMaxsize(int maxsize) { m_maxsize = maxsize; }
        int threshold() { return m_threshold; }
        void setThreshold(int threshold) { m_threshold = threshold; }
        QString & type() { return m_type; }
        void setType(const QString& type ) { m_type = type; }
        void addValue(const QString&);
};

class IndexTheme
{
    QList<Group> m_groups;
    QString m_path;
    QStringList m_inherits;
    QHash<int, QStringList> m_matchesCache;
    int m_test;
    
    public:
        IndexTheme(const QString&);
        IndexTheme();
        ~IndexTheme(){}
        QString name() { QDir dir(m_path); return dir.dirName();  }
        void setPath(const QString& path);
        QStringList& match(int);
        //QString lastMatch(const QString& iconName, int iconSize);
        //QString firstMatch(int);
        QStringList inherits() { return m_inherits; }
        QString path() { return m_path; }
        bool isEmpty() { return m_groups.isEmpty(); }
        bool operator==(IndexTheme&);
        bool operator==(const QString&);
        QList<Group>& groups() { return m_groups; }
        void setTest(int test) { m_test = test; }
        int test() {return m_test;}
};

class FileIconProvider : public QFileIconProvider
{

    QList<QDir> m_baseDirs;
    IndexTheme * m_indexTheme;
    QStringList m_filters;
    QString m_iconTheme;
    static QHash<QString, QIcon> m_iconsCache;
    static QHash<QString, QIcon> m_thumbsCache;
    static QFreeDesktopMime m_mime;
    QHash<QString, QString> m_mimeIcons;
    QStringList m_imageExt;
    QSize m_iconSize;
    bool m_showThumnails;
    
    public:
        FileIconProvider();
        ~FileIconProvider(){ }
        QIcon icon(const QFileInfo &) const;
        void setIconSize(QSize);
        void setIconSize(int, int);
        void showThumbnails(bool);
    
    protected:
        void loadMimeTypes();
        QList<IndexTheme*> m_indexThemeCache;
        QHash<QString, QStringList> m_dirsCache;
        QHash<QString, QFileInfoList> m_dirsCache2;
        QIcon _getIcon(const QString& name, int size=64) const;
        QString lookupFallbackIcon(const QString&);
        QString lookupIcon(const QString& name, const QString& theme, int size);
        QList<QDir> getBaseDirectories();
        IndexTheme* getIndexThemeData(const QString& );
        QString getPathParts(const QString&, int);
        QString getIconPathInDir(const QString&, const QString&);
};

#endif
