/* 
 *  Copyright (c) 2010 Martin Ribelotta
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "imagerenderer.h"

ImageRenderer::ImageRenderer(const QSize &size, const QString &path, QObject *parent, const char *member) :
    QObject(parent)
{
    this->size = size;
    this->path = path;
    parent->connect(this, SIGNAL(finish(QImage*)),
            parent, member,
            Qt::QueuedConnection);
}

static inline bool needScale(const QSize& container, const QSize& contained)
{
    return (contained.width()>container.width() ||
            container.height()>container.height());
}

void ImageRenderer::run()
{
    QImage image(path);
    if (!path.isEmpty()) {
        if (!needScale(size, image.size()))
            emit finish(new QImage(image));
        else
            emit finish(new QImage(image.scaled(size, Qt::KeepAspectRatio,
                                            Qt::SmoothTransformation)));
    }
}
