/* 
 *  Copyright (c) 2010 Martin Ribelotta
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IMAGERENDERER_H
#define IMAGERENDERER_H

#include <QObject>
#include <QRunnable>
#include <QImage>

/**
  Read and scale (if necesary) image to specific container size
 */
class ImageRenderer : public QObject, public QRunnable
{
Q_OBJECT
public:
    /**
      Construct image renderer for image into "path" with maximum size of "size"<br>
      Notify to "parent"->"member"<br>
      Member must be receive a QImage* parameter

      @param size Size of the container
      @param path Path of image to preview
      @param parent Objeto to notify the result
      @param member SLOT with QImage* param to receive result
     */
    explicit ImageRenderer(const QSize& size, const QString& path, QObject *parent, const char *member);

    /**
      Process image read and scaling (if necesary) in a thread pool
     */
    virtual void run();

signals:
    /**
      Emited when process finished<br>
      You need destroy a image parameter manualy with "delete image"

      @param image Pointer to dynamic image
     */
    void finish(QImage* image);

private:
    QString path;
    QSize size;
};

#endif // IMAGERENDERER_H
