/* 
 *  Copyright (c) 2010 Martin Ribelotta
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PREVIEWWIDGET_H
#define PREVIEWWIDGET_H

#include "ui_previewwidget.h"
#include "iconset.hpp"
#include <QWidget>
#include <QFileInfo>

namespace Ui {
    class PreviewWidget;
}

class QThreadPool;
class QFileIconProvider;

/**
  <p>Widget to preview file</p>
  <p>Only preview images and folder information</p>
 */
class PreviewWidget : public QWidget {
    Q_OBJECT
public:
    PreviewWidget(QWidget *parent = 0);
    ~PreviewWidget();

public slots:
    /**
      Init previsualization of file defined by "info"
      @param info File to previsualized
     */
    void startPreview(const QFileInfo& info);

    /**
      Show formated file information as text
      @param info File to preview
     */
    void showFileInfo(const QFileInfo& info);

protected:
    void changeEvent(QEvent *e);

private:
    Ui::PreviewWidget *ui;
    QThreadPool *pool;
    QFileIconProvider *iconProvider;

private slots:
    void finished(QImage *tmpImage);
};

#endif // PREVIEWWIDGET_H
