#include "scope.hpp"
#include "systemdir.hpp"
//#include "toolbarsearch.h"
#include "searchlineedit.h"

#include <QtCore/QDir>
#include <QtCore/QList>
#include <QtCore/QModelIndex>
#include <QtCore/QString>
//#include <QtCore/QMimeData>
#include <QtCore/QTextStream>
#include <QtCore/QList>

//#include <QtGui/QDrag>
#include <QtGui/QIcon>
#include <QtGui/QListView>
#include <QtGui/QMainWindow>
#include <QtGui/QMessageBox>
//#include <QtGui/QMouseEvent>
#include <QtGui/QWidget>
#include <QtGui/QInputDialog>
#include <QProcess>
#include <QDesktopServices>
#include <QUrl>
#include <QShortcut>
#include <QMimeData>
#include <iostream>
#include <QSlider>
#include <QDebug>
// Status messages delay.
quint16 Scope::s_messageDelay (3000);

//==============================================================================
//= Builders                                                                   =
//==============================================================================


Scope::Scope(const QString & p_path,
             QWidget       * p_parent)
: QMainWindow(p_parent),
  m_views(this),
  m_bookmarks(this),
  m_hiddenFiles(true),
  m_hiddenFilesWhenFiltering(false)
{

    setupInterface();
    loadSettings();
    m_tabWidget = NULL;
    
    // Signals and slots for some behavior stuff.
    connect(&m_filesystem, SIGNAL(rootPathChanged(const QString &)),
                           SLOT(loadFiles(const QString &)));
    connect(&m_filesystem, SIGNAL(rowsInserted(const QModelIndex &,
                                               int,
                                               int)),
                           SLOT(updateCount()));
    connect(&m_filesystem, SIGNAL(rowsRemoved(const QModelIndex &,
                                              int,
                                              int)),
                           SLOT(updateCount()));
                           
    //connect(m_ui.t_filter, SIGNAL(textChanged(const QString &)),
    //                       SLOT(loadFiles()));
    connect(&m_bookmarks,  SIGNAL(triggered(QAction *)),
                           SLOT(goFromBookmark(QAction *)));
                           
    connect(m_ui.a_showThumbnails, SIGNAL(toggled(bool)), &m_filesystem, SLOT(showThumbnails(bool)));
    
    
    m_filesystem.setNameFilterDisables(false);
    setupDefaultView(m_ui.v_files);
    
    
    pstart = false;
    
    if (p_path.isEmpty())
        setDir(dir());
    else
        setDir(p_path);
        
    //start history actions
    m_historyBackAction->setEnabled(false);
    m_historyForwardAction->setEnabled(false);
    backActions = new QList<QAction *>;
    forwardActions = new QList<QAction *>;
        
    m_ui.a_hiddenFiles->setChecked(m_hiddenFiles);
    
    m_ui.previewWidget->startPreview(QFileInfo(dir()));
    
    //m_qtermWidget = NULL;
    m_qtermWidget = new QTermWidget(0, this);
    connect(m_qtermWidget, SIGNAL(receivedData(const QString&)), this, SLOT(test(const QString&)));

    m_stackedView = new QStackedWidget(m_ui.centralwidget);
    m_stackedView->addWidget(m_ui.v_files);
    if ( m_qtermWidget )
        m_stackedView->addWidget(m_qtermWidget);
    
    QLayout *layout = m_ui.centralwidget->layout();
    layout->removeWidget(m_ui.v_files);
    layout->addWidget(m_stackedView);
    
    show();
}

void Scope::loadSettings()
{
    m_settings = new QSettings("scope", "scope", this);
    
    foreach(QString key, m_settings->allKeys())
    {
        if ( key.contains("Bookmarks") )
            loadBookmark(m_settings->value(key).toString());
    }
    
    if ( m_settings->contains("ShowHiddenFiles") )
        m_hiddenFiles = m_settings->value("ShowHiddenFiles").toBool();
    
    if ( m_settings->contains("ShowHiddenFilesWhenFiltering") )
        m_hiddenFilesWhenFiltering = m_settings->value("ShowHiddenFilesWhenFiltering").toBool();
        
    if ( m_settings->contains("ShowThumbnails") )
    {
        m_ui.a_showThumbnails->setChecked(m_settings->value("ShowThumbnails").toBool());
        m_filesystem.showThumbnails(m_ui.a_showThumbnails->isChecked());
    }
    
    if ( m_settings->contains("ShowFilter") )
        m_ui.a_filter->setChecked(m_settings->value("ShowFilter").toBool());
    
    
}

void Scope::saveSettings()
{
     m_settings->setValue("ShowThumbnails", m_ui.a_showThumbnails->isChecked());
     m_settings->setValue("ShowFilter", m_ui.a_filter->isChecked());
}


//==============================================================================
//= Getters and setters                                                        =
//==============================================================================


QString Scope::dir()
{
    QListView * view = getCurrentView();
    QString path = m_filesystem.filePath(view->rootIndex());
    
    if ( ! path.isEmpty() )
        return path;
    
    return QDir::currentPath();
}


// setDirectory:
//   Change the current directory.
//   Parameters:
//       p_path -> directory path to change to.
bool Scope::setDir(const QString & p_path)
{
    if (QDir(p_path).exists())
    {
        if(pstart)
        {
            if (!(m_historyBackAction->isEnabled()))
                m_historyBackAction->setEnabled(true);
        
            QAction *backAction = new QAction(dir(), this);
            backAction->setData(backActions->length());
            backActions->prepend(backAction);
            
            //we clear the menu each time a new directory is opened
            //and recreate it using the backActions list which has the right order
            //Reason: The menu.addAction() always puts the most recent QAction in the end
            //and it should be on the beginning
            if(!(m_historyBackMenu->isEmpty()))
                m_historyBackMenu->clear();
                
            m_historyBackMenu->addActions(*backActions);

            
            /*foreach (QAction *action, *backActions)
            {
                action->setText(QDir(action->text()).dirName());
                m_historyBackMenu->addAction(action);
            }*/
        }
        
        qDebug() << Q_FUNC_INFO;
        
        // Change to the given directory.
        m_filesystem.setRootPath(p_path);
        QDir::setCurrent(p_path);
        m_path = dir();
        pstart = true;
        pathline->insertText(m_path);
        return true;
    }
    
    return false;
}


//==============================================================================
//= Inline methods                                                             =
//==============================================================================


// showMessage:
//   Show a temporary message in the status bar.
inline void Scope::showMessage(const QString & p_message)
{
    m_ui.status->showMessage(p_message, s_messageDelay);
}


//==============================================================================
//= Other methods                                                              =
//==============================================================================


/*void Scope::mouseMoveEvent(QMouseEvent * p_event)
{
    qDebug("mouseMoveEvent.");
    if (p_event->button() != Qt::LeftButton)
    {
        return;
    }
    if ((p_event->pos() - m_dragStartPos).manhattanLength()
        < QApplication::startDragDistance())
    {
        return;
    }
    
    qDebug("Dragging...");
    QDrag     * drag     = new QDrag(this);
    QMimeData * mimeData = new QMimeData;
    
    QWidget * child(childAt(p_event->pos()));
    if (child == m_ui.v_files)
    {
        qDebug("from the list :D");
    }
    mimeData->setText("foobar");
    drag->setMimeData(mimeData);
    
    Qt::DropAction dropAction = drag->exec(Qt::CopyAction | Qt::MoveAction);
}


void Scope::mousePressEvent(QMouseEvent * p_event)
{
    qDebug("mousePressEvent.");
    if (p_event->button() == Qt::LeftButton)
    {
        m_dragStartPos = p_event->pos();
    }
}*/


// setupInterface:
//   Load and set up the user interface.
void Scope::setupInterface()
{
    // Set up user interface.
    m_ui.setupUi(this);
    
    setupToolBar();
    
    // Add widgets to the status bar.  That should be built into the .ui file
    // but I could not achieve that.
    //statusBar()->addWidget(m_ui.l_fileCount);
    
    status = new QLabel();  //the content will be updated after
    
    m_zoomWidget = new QWidget(this);
    
    //Zoom Slider
    m_zoomSlider = new QSlider(Qt::Horizontal, m_zoomWidget);
    m_zoomSlider->setGeometry(QRect(80, 200, 160, 19));
    m_zoomSlider->setTickInterval(10);
    m_zoomSlider->setPageStep(1);
    
    const int min = ZoomLevel::minimumLevel();
    const int max = ZoomLevel::maximumLevel();
    
    m_zoomSlider->setRange(min, max);
    m_zoomSlider->setValue(ZoomLevel::zoomLevelForIconSize(m_filesystem.getIconSize().width())); //ToDo:use settings value instead of this static value
    
    connect(m_zoomSlider, SIGNAL(valueChanged(int)), this, SLOT(setZoomLevel(int)));
    
    //Zoom buttons
    //m_zoomIn = new QPushButton(this);
    m_zoomIn = new QToolButton(m_zoomWidget);
    m_zoomIn->setAutoRaise(true);
    m_zoomOut = new QToolButton(m_zoomWidget);
    m_zoomOut->setAutoRaise(true);
    
    m_zoomIn->setIcon(m_icons.getIcon("16x16/actions/zoom-in.png"));
    m_zoomOut->setIcon(m_icons.getIcon("16x16/actions/zoom-out.png"));
            
    connect(m_zoomIn, SIGNAL(clicked()), this, SLOT(zoomIn()));
    connect(m_zoomOut, SIGNAL(clicked()), this, SLOT(zoomOut()));
    

    //Add the widgets to statusBar
    QHBoxLayout *zoomWidgetLayout = new QHBoxLayout(m_zoomWidget);
    zoomWidgetLayout->setSpacing(0);
    zoomWidgetLayout->setMargin(0);
    zoomWidgetLayout->addWidget(m_zoomOut);
    zoomWidgetLayout->addWidget(m_zoomSlider);
    zoomWidgetLayout->addWidget(m_zoomIn);
    
    m_zoomWidget->setFixedSize(150, 20);
    
    
    m_ui.status->addWidget(status);
    statusBar()->addPermanentWidget(m_zoomWidget);

    
    // Use an action group for the view mode actions.
    m_views.addAction(m_ui.a_iconView);
    m_views.addAction(m_ui.a_listView);
    m_views.addAction(m_ui.a_terminalView);
    
    

    //connect(m_ui.t_dir,    SIGNAL(returnPressed()), SLOT(click()));
    
    //  File menu.
    connect(m_ui.a_createFolder, SIGNAL(activated()), 
                                 SLOT(createFolder()));

    m_ui.a_createFolder->setShortcut(tr("Ctrl+N"));
    
    connect(m_ui.a_quit,         SIGNAL(activated()),
            this,                SLOT(close()));
    //  Edit menu.
    connect(m_ui.a_remove,    SIGNAL(activated()),
                              SLOT(remove()));
    connect(m_ui.a_rename,    SIGNAL(activated()), 
                              SLOT(rename()));
    connect(m_ui.a_selectAll, SIGNAL(activated()),
            m_ui.v_files,     SLOT(selectAll()));
    //  View menu.
    connect(m_ui.a_filter,      SIGNAL(toggled(bool)),
            m_toolbarSearch,      SLOT(setVisible(bool)));
    connect(m_ui.a_hiddenFiles, SIGNAL(toggled(bool)),
                                SLOT(hiddenFiles(bool)));
    connect(m_ui.a_hiddenFilesWhenFiltering, SIGNAL(toggled(bool)),
                                SLOT(hiddenFilesWhenFiltering(bool)));
    connect(&m_views,           SIGNAL(triggered(QAction *)),
            this,               SLOT(changeView()));
    //  Bookmarks menu.
    connect(m_ui.a_addBookmark, SIGNAL(activated()),
                                SLOT(addBookmark()));
    //  Help menu.
    connect(m_ui.a_about,        SIGNAL(activated()),
            &m_about,            SLOT(show()));
    connect(m_ui.a_aboutDelphos, SIGNAL(activated()),
            &m_aboutDelphos,     SLOT(show()));
    
    connect(m_ui.a_createFile, SIGNAL(activated()), this, SLOT(createFile()));
    //connect(m_ui.a_terminalView, SIGNAL(activated()),
    //        this, SLOT(switchToTerminal));

    connect(m_ui.a_createTab, SIGNAL(activated()), this, SLOT(addNewTab()));

    //connect(m_ui.c_prevDirs, SIGNAL(activated(int)),
    //                   this,SLOT(goToDir(int)));
     
         
    pstart = false;
            
    //Open file action
    a_open = new QAction( tr( "Open" ), this );
    connect(a_open,              SIGNAL(activated()), this,
                                 SLOT(openFile()));
    //Copy file action                             
    a_copy = new QAction( tr( "Copy" ), this );
    a_copy->setShortcut(tr("Ctrl+C"));
    m_ui.v_files->addAction(a_copy);
    connect(a_copy,              SIGNAL(activated()), this,
                                 SLOT(copy()));
                                 
    //cut file action
    a_cut = new QAction(tr( "Cut" ), this);
    a_cut->setShortcut(tr("Ctrl+X"));
    m_ui.v_files->addAction(a_cut);
    connect(a_cut,              SIGNAL(triggered()),this,
                                SLOT(cut()));
    
    //Paste file action                             
    a_paste = new QAction( tr( "Paste" ), this );
    a_paste->setShortcut(tr("Ctrl+V"));
    m_ui.v_files->addAction(a_paste);
    connect(a_paste,   SIGNAL(activated()), this,
                                  SLOT(paste()));
                                  
    //Properties file action
    a_properties = new QAction( tr("Properties"), this);
    
    connect(a_properties, SIGNAL(activated()), this,
                            SLOT(showProperties()));
                            //SLOT(show_properties()));
                        
   //QFileInfo archiveInfo =    QFileInfo (m_icons.getIconPath(
   //"16x16/actions/archive.png"));
                      
    // Icons.
    m_ui.a_quit->setIcon(m_icons.getIcon(
        "16x16/actions/application-exit.png"));
    m_ui.a_remove->setIcon(m_icons.getIcon(
        "16x16/actions/archive-remove.png"));
    m_ui.a_addBookmark->setIcon(m_icons.getIcon(
        "16x16/actions/bookmark-new.png"));
    m_ui.a_createFolder->setIcon(m_icons.getIcon(
        "16x16/actions/folder-new"));
    m_ui.a_createFile->setIcon(m_icons.getIcon(
        "16x16/mimetypes/text-x-generic"));
    a_open->setIcon(m_icons.getIcon(
        "16x16/actions/document-open.png"));
    a_copy->setIcon(m_icons.getIcon(
        "16x16/actions/edit-copy"));
    a_paste->setIcon(m_icons.getIcon(
        "16x16/actions/edit-paste"));
    a_cut->setIcon(m_icons.getIcon(
        "16x16/actions/edit-cut"));

        

    
}

//Called everytime the selection changes, 
//but is only used when the view is clicked and no item is selected
void Scope::onSelectionChanged(const QItemSelection & selected, const QItemSelection & deselected)
{
    if ( selected.indexes().isEmpty() )
    {
        QFileInfo file_info(dir());
        m_ui.previewWidget->startPreview(file_info);
    }
}

//Called when some item is clicked on the view
void Scope::viewClicked(const QModelIndex & index)
{
    QFileInfo fileInfo(dir() + QDir::separator() + index.data().toString());
    m_ui.previewWidget->startPreview(fileInfo);
    
}

void Scope::zoomOut()
{
   const int value = m_zoomSlider->value();
   m_zoomSlider->setValue(value - 1);
}

void Scope::zoomIn()
{
    const int value = m_zoomSlider->value();
    m_zoomSlider->setValue(value + 1);
}

void Scope::setZoomLevel(int zoomLevel)
{
    int iconZoom = ZoomLevel::iconSizeForZoomLevel(zoomLevel);
    QListView * currentView = getCurrentView();
    qDebug() << iconZoom;
    currentView->setGridSize( QSize(iconZoom+WIDTH_GRID_SPACE, iconZoom+HEIGHT_GRID_SPACE) );
    currentView->setIconSize(QSize(iconZoom, iconZoom));
    updateZoomSliderToolTip(iconZoom);
    m_filesystem.setIconSize(iconZoom, iconZoom);
}

void Scope::updateZoomSliderToolTip(int zoomLevel)
{
    m_zoomSlider->setToolTip(QString("Size: %1 pixels").arg(zoomLevel));
}

void Scope::goBack(QAction *b_action)
{
    
    //activates the ForwardAction when backAction is first used
    if(!m_historyForwardAction->isEnabled())
        m_historyForwardAction->setEnabled(true);
        
    //Saves the current folder in the forward menu
    QAction *forwardAction = new QAction(dir(), this);
    forwardAction->setData(forwardActions->length());
    forwardActions->prepend(forwardAction);
        
    //If back button was pressed
    if (b_action == NULL)
    {
        b_action = new QAction(backActions->first()->text(), this);
        
        backActions->removeFirst();
    }
    else //If some action was pressed in the Menu
    {
        int removeTo = b_action->data().toInt();
        
        for (int i = backActions->length()-1; i >= removeTo; i--)
        {
            if (i != removeTo) //so it doesn't save the last
            { 
                backActions->first()->setData(forwardActions->length());
                forwardActions->prepend(backActions->first());
            }
            
            backActions->removeFirst();
        }
    }
    
    pstart = false;
    
    //recreate the menus
    if(!(m_historyBackMenu->isEmpty()))
        m_historyBackMenu->clear();
    if(!(m_historyForwardMenu->isEmpty()))
        m_historyForwardMenu->clear();
    
    if(backActions->isEmpty())
        m_historyBackAction->setEnabled(false);
    else
        m_historyBackMenu->addActions(*backActions);
        
    m_historyForwardMenu->addActions(*forwardActions);
    
    //go to the directory
    goManually(b_action->text());
}

void Scope::goForward(QAction *f_action)
{
    //activates the ForwardAction when backAction is first used
    if(!m_historyBackAction->isEnabled())
        m_historyBackAction->setEnabled(true);
        
    //Saves the current folder in the forward menu
    QAction *backAction = new QAction(dir(), this);
    backAction->setData(backActions->length());
    backActions->prepend(backAction);
        
    //If back button was pressed
    if (f_action == NULL)
    {
        f_action = new QAction(forwardActions->first()->text(), this);
        
        forwardActions->removeFirst();
    }
    else //If some action was pressed in the Menu
    {
        int removeTo = f_action->data().toInt();
        
        for (int i = forwardActions->length()-1; i >= removeTo; i--)
        {
            if (i != removeTo) //so it doesn't save the last
            {
                forwardActions->first()->setData(backActions->length());
                backActions->prepend(forwardActions->first());
            }
            forwardActions->removeFirst();
        }
    }
    
    pstart = false;
    
    //recreate the menus
    if(!(m_historyBackMenu->isEmpty()))
        m_historyBackMenu->clear();
    if(!(m_historyForwardMenu->isEmpty()))
        m_historyForwardMenu->clear();
    
    if(forwardActions->isEmpty())
        m_historyForwardAction->setEnabled(false);
    else
        m_historyForwardMenu->addActions(*forwardActions);
        
    m_historyBackMenu->addActions(*backActions);
    
    goManually(f_action->text());
}

void Scope::setupToolBar()
{

    //backward button and menu
    m_historyBackAction = new QAction(this);
    m_historyBackAction->setIcon(style()->standardIcon(QStyle::SP_ArrowBack, 0, this));
    m_historyBackMenu = new QMenu(this);
    m_historyBackAction->setMenu(m_historyBackMenu);
            
    //the button
    connect(m_historyBackAction, SIGNAL(triggered()),this,
                                SLOT(goBack()));
            
    //the menu
    connect(m_historyBackMenu, SIGNAL(triggered(QAction *)),
            this, SLOT(goBack(QAction*)));
            
    m_ui.m_navigationBar->addAction(m_historyBackAction);

    //forward button and menu
    m_historyForwardAction = new QAction(this);
    m_historyForwardAction->setIcon(style()->standardIcon(QStyle::SP_ArrowForward, 0, this));
    m_historyForwardMenu = new QMenu(this);

    connect(m_historyForwardAction, SIGNAL(triggered()),
            this, SLOT(goForward()));

    connect(m_historyForwardMenu, SIGNAL(triggered(QAction *)),
            this, SLOT(goForward(QAction *)));
            
    m_historyForwardAction->setMenu(m_historyForwardMenu);
    m_ui.m_navigationBar->addAction(m_historyForwardAction);
    
    //up button
    a_goUp = new QAction(this);
    a_goUp->setIcon(style()->standardIcon(QStyle::QStyle::SP_ArrowUp, 0, this));
    connect(a_goUp, SIGNAL(triggered()), this,
                        SLOT(goUp()));
    m_ui.m_navigationBar->addAction(a_goUp);
    
    //home button
    a_goHome = new QAction(this);
    a_goHome->setIcon(style()->standardIcon(QStyle::SP_DirHomeIcon, 0, this));
    connect(a_goHome, SIGNAL(triggered()), this,
                    SLOT(goHome()));
    m_ui.m_navigationBar->addAction(a_goHome);
    
    //Path bar
    m_navigationSplitter = new QSplitter(m_ui.m_navigationBar);
    //m_navigationSplitter->addWidget(m_tabWidget->locationBarStack());
    pathline = new PathLine(&m_filesystem);
    m_navigationSplitter->addWidget(pathline);
    //m_navigationSplitter->addWidget(m_ui.t_dir);
    //m_ui.t_dir = new PathLine(&m_filesystem);

    //Search bar
    m_toolbarSearch = new SearchLineEdit(m_ui.m_navigationBar);
    connect(m_toolbarSearch, SIGNAL(textChanged(QString)),this,
                    SLOT(filter(QString)));
    
    a_focusSearch = new QAction(this);
    a_focusSearch->setShortcut(Qt::CTRL | Qt::Key_F);
    m_ui.v_files->addAction(a_focusSearch);
    connect(a_focusSearch,   SIGNAL(triggered()), this,
                                  SLOT(setFocusOnSearch()));           
    
    m_navigationSplitter->addWidget(m_toolbarSearch);
    
    //m_navigationSplitter->addWidget(m_ui.t_filter);
    
    //connect(m_toolbarSearch, SIGNAL(search(const QUrl&)),
    //        m_tabWidget, SLOT(loadUrl(const QUrl&)));
    m_navigationSplitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
    //m_tabWidget->locationBarStack()->setMinimumWidth(120);
    m_navigationSplitter->setCollapsible(0, false);
    m_ui.m_navigationBar->addWidget(m_navigationSplitter);
    int splitterWidth = m_navigationSplitter->width();
    QList<int> sizes;
    sizes << (int)((double)splitterWidth * .80) << (int)((double)splitterWidth * .20);
    m_navigationSplitter->setSizes(sizes);
}

void Scope::setFocusOnSearch()
{
    m_toolbarSearch->setFocus(Qt::ShortcutFocusReason);
}


/*void Scope::goToDir(int pos)
{   
    
    setDir(m_ui.c_prevDirs->itemText(pos));
    //QTextStream(stdout) << pos << " " << m_ui.c_prevDirs->count();
    if (pos == (m_ui.c_prevDirs->count())-2)
    {
        m_ui.c_prevDirs->setEnabled(false);
        m_ui.b_goBack->setEnabled(false);
        m_ui.c_prevDirs->clear();
    }
    else
        for (int i=0; i <= pos; i++)
            m_ui.c_prevDirs->removeItem(0);

    
}*/

void Scope::showProperties()
{
    
    QModelIndex index (getCurrentView()->currentIndex());
    QList<QModelIndex> indexes (m_selector->selectedIndexes());
    
    if( ! m_selector->hasSelection() ) //if it's the root directory
        indexes << m_filesystem.index(QDir::currentPath());
    
    m_properties = new PropertiesDialog(indexes, m_filesystem);
    
    if(m_properties->exec() == QDialog::Accepted && m_properties->isSettingChanged())
        m_properties->applyChanges();

    
    delete m_properties;
    m_properties = NULL;
}

void Scope::goHome()
{
    //m_filesystem.setDirectory(QDir::homePath());
    setDir(QDir::homePath());
}

void Scope::goUp()
{
    QDir dir(QDir::current());

    dir.cdUp();
    if (dir.isRoot())
        a_goUp->setDisabled(true);

    //QTextStream(stdout) << dir.absolutePath() + "\n";
    //m_filesystem.setDirectory(dir.absolutePath());
    setDir(dir.absolutePath());
}


void Scope::showItemContextMenu(const QPoint & p_coordinates)
{
    QList<QModelIndex> indexes (m_selector->selectedIndexes());
    // Display a context menu for the items in the list.
    QMenu       menu  (this);
    QMenu       createMenu(tr("Create New"), this);
    QModelIndex index (getCurrentView()->currentIndex());
    const QMimeData *mime = QApplication::clipboard()->mimeData();
    
    
    menu.addSeparator()->setText(index.data().toString());
    
    if (indexes.isEmpty())
    {
        createMenu.addAction(m_ui.a_createFolder);
        createMenu.addAction(m_ui.a_createFile);
        menu.addMenu(&createMenu);
        
        if ( mime->hasUrls() ) 
            menu.addAction(a_paste);
        
        menu.addAction(a_properties);
    }
    else
    {
        menu.addAction(a_open);
        menu.addSeparator();
        menu.addAction(a_copy);
        menu.addAction(a_cut);
        menu.addSeparator();
        menu.addAction(m_ui.a_remove);
        menu.addAction(m_ui.a_rename);
        menu.addSeparator();
        menu.addAction(a_properties);
    }
    
    menu.exec(getCurrentView()->mapToGlobal(p_coordinates));
    
}

QList<QUrl> Scope::copy()
{
    QList<QModelIndex> indexes (m_selector->selectedIndexes());
    QFileInfo file_info;
    QString current_dir (dir());
    QString separator(QDir::separator());
    QList<QUrl> urlList;
    QMimeData *mime = new QMimeData();
    
    foreach(QModelIndex index, indexes)
    {
        file_info = current_dir + separator + index.data().toString();
        urlList.append(QUrl::fromLocalFile(file_info.canonicalFilePath()));
    }
    
    mime->setUrls(urlList);
    QApplication::clipboard()->setMimeData(mime);
    last_action = ActionCopy;
    
    //delete mime;
    return urlList;
}

void Scope::cut()
{
    m_filesystem.clearCutFiles();
    QList<QUrl> urlList;
    urlList = copy();
    m_filesystem.setCutFilesList(urlList);
    
    last_action = ActionCut;
}

void Scope::paste()
{
    
    m_filesystem.clearCutFiles();
    
    const QMimeData *mime = QApplication::clipboard()->mimeData();
    if (mime && mime->hasUrls()) {
        QString destinationPath = dir();
        QStringList fileList;
        foreach(QUrl url, mime->urls())
            fileList.append(url.toLocalFile());
        QtFileCopier fileCopier;
        QtCopyDialog d(&fileCopier, this);
        
        switch(last_action) {
            case ActionCopy:
                fileCopier.copyFiles(fileList, destinationPath);
                break;
            case ActionCut:
                fileCopier.moveFiles(fileList, destinationPath);
                break;
            default:
                break;
        }
        
        d.exec();
    }
    
    //delete mime;
    
    //update the view
    //loadFiles();
}

void Scope::openFile(const QModelIndex& index)
{
        QModelIndex validIndex = index;
        
        if ( ! index.isValid() )
            validIndex = QModelIndex(getCurrentView()->currentIndex());
        
        if ( m_filesystem.fileInfo(validIndex).isDir() )
            return;
            
        QDesktopServices::openUrl(QUrl(validIndex.data().toString(), QUrl::TolerantMode));
}


// addBookmark:
//   Add the current directory as item in the Bookmarks menu.
void Scope::loadBookmark(const QString& path)
{
    if ( path.isEmpty() )
        return;
    
    QAction * action = new QAction(m_icons.getIcon("16x16/places/folder.png"),
                                   path,
                                   &m_bookmarks);
    m_ui.n_bookmarks->addAction(action);
    showMessage(tr("Bookmark added."));
}

void Scope::addBookmark(const QString& path)
{   
    QString _path = path;
    if ( _path.isEmpty() )
        _path = m_path;
    if ( _path.isEmpty() )
        return;
        
    loadBookmark(_path);
    m_settings->setValue("Bookmarks/" + QFileInfo(_path).fileName(), _path);
}


// changeView:
//   Change the list view mode.
void Scope::changeView()
{
    QStackedWidget * stackedView = 0;
    
    if ( m_qtermWidget )
    qDebug() << m_qtermWidget->getOutputFromCommand("pwd");
    
    if ( m_tabWidget )
    {
        if ( m_tabWidget->currentIndex() < m_stackedViews.size() )
            stackedView = m_stackedViews[m_tabWidget->currentIndex()];
            
        if ( ! stackedView ) return;
    }
    else
        stackedView = m_stackedView;
    
    QListView *view = qobject_cast<QListView*>(stackedView->widget(0));
    QTermWidget *terminal = qobject_cast<QTermWidget*>(stackedView->widget(1));
    
    QAction * action = m_views.checkedAction();
    if (action == m_ui.a_iconView)
    {
        stackedView->setCurrentIndex(0);
        view->setViewMode(QListView::IconMode);
    }
    else if (action == m_ui.a_listView)
    {
        stackedView->setCurrentIndex(0);
        view->setViewMode(QListView::ListMode);
		//set the grid size back to default, 
        //because items listed don't need to be spaced between them
		view->setGridSize(QSize(-1, -1)); 
    }
    else
    {
        //switch to terminal view
        terminal->setInitialWorkingDirectory(dir());
        terminal->startShellProgram();
        stackedView->setCurrentIndex(1);
    }
}

void Scope::test(const QString& p)
{
    QString path("");
    QStringList lines = p.split("\n");
        
    if ( lines.size() >= 2 )
        path = lines[1].simplified();
        
    qDebug() << "CHANGE PATH FROM TERMINAL";
    qDebug() << path;
    setDir(path);
}


void Scope::filter(QString pattern_filter)
{
    
    QDir::Filters flags(QDir::AllEntries | QDir::System | QDir::NoDotAndDotDot);
    QStringList filters;
    
    if ( m_hiddenFilesWhenFiltering || m_hiddenFiles )
        flags |= QDir::Hidden;
    
    if ( ! pattern_filter.isEmpty() ) 
    {

        m_filesystem.setFilter(flags);
        filters << "*" + m_toolbarSearch->text().replace(' ', '*') + "*";
    }
    else{
        m_filesystem.setFilter(flags);
        loadFiles();
    }
    
    m_filesystem.setNameFilters(filters);
}

// loadFiles:
//   Load files in current directory to list widget.
//   Directories end with '/'.
//   Symbolic links end with '@'.
bool Scope::loadFiles(const QString & p_path)
{
    m_path = dir();
    
    QString path (p_path);
    if (path.isEmpty())
        path = m_path;
        
    QDir currentDir(path);
    QStringList filterlist, m_files;
    QListView *view = getCurrentView();
        
    if ( m_tabWidget ) m_tabWidget->setTabText(m_tabWidget->currentIndex(), QDir(path).dirName());
    
    if (!(currentDir.isRoot()) && !(a_goUp->isEnabled()))
        a_goUp->setDisabled(false);
    
    // Default flags.
    QDir::Filters flags (QDir::AllEntries |
                         QDir::System |
                         QDir::NoDotAndDotDot);
                         
    if (m_hiddenFiles)
        flags |= QDir::Hidden; // Add flag for hidden files.
    
    m_filesystem.setFilter(flags);
    
    view->setRootIndex(m_filesystem.index(path));
    
    //reset the filter search
    m_toolbarSearch->releaseKeyboard();
    m_toolbarSearch->clear();
    
    //change the folder details on the preview panel
    m_ui.previewWidget->startPreview(QFileInfo(path));
    
    return true;
}


// goManually:
//   Description needed.
void Scope::goManually(const QString & dir)
{
    if (dir.isEmpty ())
        setDir(pathline->path);
    else
        setDir(dir);
}


void Scope::goFromBookmark(QAction * p_action)
{
    setDir(p_action->text());
}


// goFromItem:
//   Description needed.
void Scope::goFromItem(const QModelIndex & p_index)
{
    setDir(m_filesystem.filePath(p_index));
}


// hiddenFiles:
//   Show or hide the hidden files.
void Scope::hiddenFiles(bool p_value)
{
    m_hiddenFiles = p_value;
    m_settings->setValue("ShowHiddenFiles", p_value);
    loadFiles();
}

void Scope::hiddenFilesWhenFiltering(bool p_value)
{
    m_hiddenFilesWhenFiltering = p_value;
    m_settings->setValue("ShowHiddenFilesWhenFiltering", p_value);
}

void Scope::createFile()
{
    bool    ok;
    QString fileName (QInputDialog::getText(this,
                                           tr("Create file"),
                                           tr("File's name:"),
                                           QLineEdit::Normal,
                                           "",
                                           &ok));
                                           
    if ( ! ok )
        return;
    
    QDir::setCurrent(dir());
    QFile file(fileName);
    file.open(QIODevice::WriteOnly);
    file.write(" ");
    file.close();
}

bool Scope::createFolder()
{ 
    bool    ok;
    QString dirName (QInputDialog::getText(this,
                                           tr("Create folder"),
                                           tr("Folder's name:"),
                                           QLineEdit::Normal,
                                           "",
                                           &ok));
    if (! ok)
    {
        // The user cancelled.
        return false;
    }
    
    if (! dirName.isEmpty() &&
        QDir::current().mkdir(dirName))
    {
        showMessage(tr("Folder \"%0\" created.")
                    .arg(dirName));
        loadFiles();
        return true;
    }
    else
    {
        QMessageBox::information(this, tr("Create folder"),
                                 tr("Failed to create the folder."));
        return false;
    }
}


bool Scope::rename(QString oldName, QString newName)
{ 

    if ( oldName.isEmpty() ){
        //if oldName isn't passed, it's presumed the user selected it
        QList<QModelIndex> indexes (m_selector->selectedIndexes());
        if (indexes.isEmpty())
            return false;
            
        QString oldName (indexes[0].data().toString());
        
        if ( newName.isEmpty() ){
            // if oldName is passed and newName is empty, ask for it
            bool    ok;
            newName = QInputDialog::getText(this,
                                                   tr("Rename"),
                                                   tr("Rename to:"),
                                                   QLineEdit::Normal,
                                                   oldName,
                                                   &ok);
            
            if (! ok) //If the user cancelled.
                return false;
        }
    } 
    
    
    if (! newName.isEmpty() &&
        QDir::current().rename(oldName, newName))
    {
        showMessage(tr("%0 renamed to %1.")
                    .arg(oldName)
                    .arg(newName));
        loadFiles();
        return true;
    }
    else
    {
        QMessageBox::information(this, tr("Rename"),
                                 tr("Failed to rename."));
        return false;
    }
}


// remove:
//   Remove a file or directory regardless of whether it is empty or not.
//   Parameters:
//       p_path -> path to file to remove
void Scope::remove()
{
    QList<QModelIndex> indexes (m_selector->selectedIndexes());
    foreach (QModelIndex index, indexes)
    {
        if (m_filesystem.isDir(index))
            m_filesystem.rmdir(index);
        else
            m_filesystem.remove(index);

        /* //old code below, maybe this one above is better :>
        
        QString path (index.data().toString());
        
        showMessage(tr("Removing %0...").arg(path));
        remove(path);*/
    }
}


bool Scope::remove(const QString & p_path)
{
    QFileInfo givenFile (p_path);
    
    if (givenFile.isFile())
    {
        // Remove file.
        if (! QDir::current().remove(p_path))
        {
            return false;
        }
    }
    
    else if (givenFile.isDir())
    {
        // Remove directory.
        QDir givenDir (p_path);
        foreach (QString itemFile, givenDir.entryList(QDir::NoDotAndDotDot))
        {
            // Remove recursively all the content.  If the directory is
            // empty, the loop automatically will not be made.
            remove(p_path + '/' + itemFile);
        }
        // Remove the already empty directory.
        if (! QDir::current().rmdir(p_path))
        {
            return false;
        }
    }
    
    // The file list is not refreshed here, instead it is refreshed in the
    // remove() method, which is the one called by the actions.
    return true;
}


void Scope::updateCount()
{
    // Update file count in the status bar.
    QDir currdir(QDir::currentPath());
    int dir_count = 0;
    int file_count = 0;
    QStringList entries = currdir.entryList(//m_filters, 
        QDir::AllDirs | 
        QDir::Drives  | 
        QDir::NoDotAndDotDot |
        QDir::Files
        );
    foreach (QString entry , entries){
        QFileInfo file_info = QFileInfo(QString("%1/%2").arg(QDir::currentPath(), entry));
        
        if (file_info.isDir())
            dir_count++;
        else
            file_count++;
                
    }
    //m_fileCount = m_filesystem.rowCount(m_filesystem.index(m_path));
    status->setText(tr("%0 files and %1 directories.")
                              .arg(QString::number(file_count),
                               QString::number(dir_count)));
    
}

void Scope::addNewTab()
{
    QLayout *layout;
    QWidget *page;
    QListView *prevView;
    
    if ( m_pages.size() == 0 )
    {
        m_tabWidget = new QTabWidget(m_ui.centralwidget);
        m_tabWidget->setTabsClosable (true);
        connect(m_tabWidget, SIGNAL(currentChanged (int)), this, SLOT(onTabChanged(int)));
        connect(m_tabWidget, SIGNAL(tabCloseRequested(int)), this, SLOT(onTabClosed(int)));
        
        layout = m_ui.centralwidget->layout();
        layout->removeWidget(m_stackedView);
        layout->addWidget(m_tabWidget);
        
        //Create page QWidget
        page = new QWidget(m_tabWidget);
        new QVBoxLayout(page);
        page->layout()->addWidget(m_stackedView);
        
        m_tabWidget->addTab(page, QDir::current().dirName());
        
        m_pages << page;
        m_stackedViews << m_stackedView;
    }
    
    if ( ! m_tabWidget )
        return;
    
    //create new page
    page = new QWidget(m_tabWidget);
    new QVBoxLayout(page);
    
    //get path of previous tab
    QString prevRootPath;
    if ( m_stackedViews.size() > 0 )
    {
        prevView = getView(m_stackedViews.size()-1);
        prevRootPath = m_filesystem.filePath(prevView->rootIndex());
    }
    else
        prevRootPath = m_filesystem.rootPath();
        
    
    //create new stacked view
    QStackedWidget * stackedView =  new QStackedWidget(page);
        
    //setup QTermWidget
    QTermWidget *termWidget = new QTermWidget(0, stackedView); 
    connect(termWidget, SIGNAL(receivedData(const QString&)), this, SLOT(test(const QString&)));
    
    //setup view's properties
    QListView *view = new QListView(stackedView);
    setupDefaultView(view);
    view->setRootIndex(m_filesystem.index(prevRootPath));

    stackedView->addWidget(view);
    stackedView->addWidget(termWidget);
    
    page->layout()->addWidget(stackedView);
    
    //add page and view to their respective lists
    m_pages << page;
    m_stackedViews << stackedView;
    
    m_tabWidget->addTab(page, QFileInfo(prevRootPath).fileName());
    m_tabWidget->setCurrentIndex(m_pages.size()-1);
    
}

QListView* Scope::getCurrentView()
{
    if ( m_tabWidget )
        return getView(m_tabWidget->currentIndex());
        
    return m_ui.v_files;
}

QListView* Scope::getView(int index)
{
    if ( index >= 0 && index < m_stackedViews.size() )
    {
        QListView *view = qobject_cast<QListView*>(m_stackedViews[index]->widget(0));
        if ( view )
            return view;
    }
    
    return m_ui.v_files;
}

QStackedWidget* Scope::getStackedView(int index)
{
    if ( index >= 0 && index < m_stackedViews.size() )
        return m_stackedViews[index];
    
    return m_stackedView;
}

void Scope::onTabChanged(int index)
{
    if ( index < m_stackedViews.size() )
    {
        QListView *view = getView(index);
        if ( ! view ) return;
        QString path = m_filesystem.filePath(view->rootIndex());
        pathline->insertText(path);
        m_selector = view->selectionModel();
        
        if ( getStackedView(index)->currentIndex() == 1 )
            m_ui.a_terminalView->setChecked(Qt::Checked);
        else
        {
            if ( view->viewMode() == QListView::IconMode )
                m_ui.a_iconView->setChecked(Qt::Checked);
            else if ( view->viewMode() == QListView::ListMode )
                m_ui.a_listView->setChecked(Qt::Checked);
        }

    }
}

void Scope::onTabClosed(int index)
{
    if ( index < m_pages.size() )
    {
        QWidget * widget = m_pages[index];
        widget->deleteLater();
        m_pages.removeAt(index);
    }
    
    if ( index < m_stackedViews.size() )
    {
        //the view has been deleted by the parent, so we only remove it from the list 
        m_stackedViews.removeAt(index);
    }

    m_tabWidget->removeTab(index);
    
    if ( m_tabWidget->count() <= 1 )
    {
        QLayout *layout = m_ui.centralwidget->layout();
        layout->removeWidget(m_tabWidget);
        layout->addWidget(m_stackedView);
        m_tabWidget->deleteLater();
        m_tabWidget = 0;
            
        if ( m_pages.size() > 0 )
            m_pages.removeFirst();
            
        if ( m_stackedViews.size() > 0 )
            m_stackedViews.removeFirst();
    }
        
}

void Scope::setupDefaultView(QListView* view)
{
    QSize iconSize = m_filesystem.getIconSize();

    view->setModel(&m_filesystem);
    view->setIconSize(iconSize);
    view->setResizeMode(QListView::Adjust);
    view->setWordWrap(true);
    view->setViewMode(QListView::IconMode);
    view->setGridSize( QSize(iconSize.width()+WIDTH_GRID_SPACE, iconSize.width()+HEIGHT_GRID_SPACE) );
    view->setSelectionRectVisible (true);
    view->setSelectionMode(QAbstractItemView::ExtendedSelection);
    
    
    connect(view, SIGNAL(activated(const QModelIndex &)),
                       SLOT(goFromItem(const QModelIndex &)));
                       
                       
    //This has to be set before emitting the signal
    view->setContextMenuPolicy(Qt::CustomContextMenu); 
    
    // Signals and slots for the GUI:
    
    connect(view,  SIGNAL(customContextMenuRequested(const QPoint&)),
                           SLOT(showItemContextMenu(const QPoint&)));
                           
    connect(view, SIGNAL(clicked (const QModelIndex & )), 
                          SLOT(viewClicked(const QModelIndex & )));
                          
    connect(view, SIGNAL(doubleClicked (const QModelIndex &)), this, SLOT(openFile(const QModelIndex&)));
                          
    QItemSelectionModel * selector = view->selectionModel();
    connect(selector, SIGNAL(selectionChanged (const QItemSelection &, const QItemSelection &)), 
                        SLOT(onSelectionChanged (const QItemSelection &, const QItemSelection &)));
                        
    m_selector = selector;
                    
}


void Scope::closeEvent(QCloseEvent * event)
{
    saveSettings();
}
