#ifndef _HPP_DELPHOS_SCOPE_
#define _HPP_DELPHOS_SCOPE_


#include "ui_scope.h"
#include "iconset.hpp"
#include "dialogs/aboutscope.hpp"
#include "dialogs/aboutdelphos.hpp"
#include "dialogs/properties.hpp"
#include "zoom_level.hpp"
#include "scope_model.hpp"
#include "path_lineedit/pathline.hpp"
#include "right_panel/previewwidget.h"
#include "qtcopydialog.h"
#include "qtfilecopier.h"
#include "qtermwidget.h"

#include <QtCore/QHash>
#include <QtCore/QModelIndex>
#include <QtCore/QPoint>
#include <QtCore/QString>
#include <QtCore/QStringList>

#include <QtGui/QAction>
#include <QtGui/QActionGroup>
#include <QtGui/QFileSystemModel>
#include <QtGui/QItemSelectionModel>
#include <QtGui/QMainWindow>
//#include <QtGui/QMouseEvent>
#include <QtGui/QWidget>
#include <QtGui/QMessageBox>
#include <QtGui/QToolBar>
#include <QtGui/QSplitter>
#include <QtGui/QFileIconProvider>
#include <QDateTime>
#include <QSortFilterProxyModel>
#include <QHBoxLayout>
#include <QToolButton>
#include <QDebug>
#include <QUrl>
#include <QClipboard>
#include <QStackedWidget>
#include <QSettings>

#define HEIGHT_GRID_SPACE 40
#define WIDTH_GRID_SPACE 80



//class ToolbarSearch;
class SearchLineEdit;

class Scope : public QMainWindow
{
    Q_OBJECT
    Q_PROPERTY(QString dir READ dir WRITE setDir)
    
public:
    
    // Builders.
    Scope(const QString & p_path   = "",
          QWidget       * p_parent = 0);
    
    // Getters and setters.
    QString dir();
    bool    setDir(const QString & p_path = "");
    
    // Inline methods.
    inline void    showMessage(const QString & p_message);
    
    // Other methods.
    void    setupInterface();
    
public slots:
    
    void addBookmark(const QString& path="");
    void changeView();
    bool createFolder();
    void goManually(const QString & dir = "");
    void goFromBookmark(QAction * p_action);
    void goFromItem(const QModelIndex & p_index);
    void hiddenFiles(bool p_value);
    void hiddenFilesWhenFiltering(bool);
    bool loadFiles(const QString & p_path = "");
    void remove();
    bool remove(const QString & p_path);
    bool rename(QString oldName="", QString newName="");
    void showItemContextMenu(const QPoint & p_coordinates);
    void updateCount();
    void openFile(const QModelIndex& index=QModelIndex());
    QList<QUrl> copy();
    void paste();
    void cut();
    void goHome();
    void goUp();
    void showProperties();
    void goBack(QAction * = 0);
    void goForward(QAction *action = 0);
    void filter(QString);
    void setupDefaultView(QListView*);
    
    //void goToDir(int);


protected:
    void closeEvent(QCloseEvent *);

/*protected:
    
    void mouseMoveEvent(QMouseEvent  * p_event);
    void mousePressEvent(QMouseEvent * p_event);*/
    
private slots:
    void setZoomLevel(int zoomLevel);
    void setFocusOnSearch();
    void zoomIn();
    void zoomOut();
    void viewClicked(const QModelIndex &);
    void onSelectionChanged(const QItemSelection &, const QItemSelection &);
    void addNewTab();
    void onTabChanged(int);
    void onTabClosed(int);
    void test(const QString& p="test");
    void createFile();
    
private:
    void setupToolBar();
    void updateZoomSliderToolTip(int zoomLevel);
    void removeTab();
    QListView* getCurrentView();
    QListView* getView(int);
    QStackedWidget* getStackedView(int);
    void loadSettings();
    void loadBookmark(const QString&);
    void saveSettings();
    
private:
    
    Ui::ScopeWindow         m_ui;
    AboutScopeDialog        m_about;
    AboutDelphosDialog      m_aboutDelphos;
    PropertiesDialog        *m_properties;
    QActionGroup            m_views;
    
    //QFileSystemModel        m_filesystem;
    QSortFilterProxyModel   *proxyModel;
    
    ScopeModel              m_filesystem;
    QItemSelectionModel *   m_selector;
    
    QString                 m_path;
    IconSet                 m_icons;
    QActionGroup            m_bookmarks;
    QStringList             m_files;
    QHash<QString, QString> m_descriptions;
    qint16                  m_fileCount;
    bool                    m_hiddenFiles;
    bool                    m_hiddenFilesWhenFiltering;
    static quint16          s_messageDelay;
    QAction                 *a_open;
    QAction                 *a_copy;
    QAction                 *a_cut;
    QAction                 *a_paste;
    QAction                 *a_properties;
    bool                    pstart;
    bool                    backUsed;
    QString                 pasteText;
    //QPoint                m_dragStartPos;
    QSplitter               *m_navigationSplitter;
    SearchLineEdit          *m_toolbarSearch;
    QMenu                   *m_historyBackMenu;
    QMenu                   *m_historyForwardMenu;
    QAction                 *a_goHome;
    QAction                 *a_goUp;
    //HistoryMenu *m_historyMenu;
    QAction                 *m_historyBackAction;
    QAction                 *m_historyForwardAction;
    QAction                 *m_historyHomeAction;
    QAction                 *m_historyRestoreLastSessionAction;
    QFileIconProvider       iconProvider;
    static QHash<QString, QIcon> iconsCache;
    QSlider                 *m_zoomSlider;
    QLabel                  *status;
    QAction                 *a_focusSearch;
    QToolButton             *m_zoomOut;
    QToolButton             *m_zoomIn;
    QWidget                 *m_zoomWidget;
    QList<QAction *>        *backActions;
    QList<QAction *>        *forwardActions;
    PathLine                *pathline;
    PreviewWidget           *previewWidget;
    QTermWidget             *m_qtermWidget;
    QTabWidget              *m_tabWidget;
    QList<QWidget*>         m_pages;
    QList<QStackedWidget*>  m_stackedViews;
    QStackedWidget *        m_stackedView;
    QSettings   *           m_settings;
    
    enum CopyCutAction_t {
        ActionNone,
        ActionCopy,
        ActionCut
    };
    CopyCutAction_t         last_action;
    
};


#endif
