/* 
 *  Copyright (c) 2010, 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "scope_model.hpp"

QPixmap * ScopeModel::m_blankPixmap = 0;

ScopeModel::ScopeModel(QObject* parent) :
    QFileSystemModel(parent),
    m_iconSize(64,64)
{
    
    m_blankPixmap = new QPixmap(m_iconSize); //48,(int)(48 * (1/ratio)));
    m_blankPixmap->fill(Qt::lightGray);
    QPainter painter(m_blankPixmap);
    painter.setPen(QPen(Qt::black, 1, Qt::DotLine));
    painter.drawRect(m_blankPixmap->rect().adjusted(0,0,-1,-1));
    painter.end();
    
    cut_files = QList<QUrl>();
    
    m_iconProvider.setIconSize(m_iconSize);

}

ScopeModel::~ScopeModel()
{
    //Clear pixmaps
    delete m_blankPixmap;
}

QSize ScopeModel::getIconSize()
{
    return m_iconSize;
}


QVariant ScopeModel::data ( const QModelIndex & index, int role) const
{
    
    if (!index.isValid())
        return QVariant();
        
    //if (index.row() >= this.m_entryList.size())
    //     return QVariant();

    int column = index.column();
 
    QFileInfo info = fileInfo(index);
    
    switch (role)
    {
        case Qt::DisplayRole:
            return info.fileName();
            
            break;
        case Qt::DecorationRole:
        
            //grey out cut files
           
            foreach( QUrl cut_file, cut_files){
                QFileInfo cut_file_info (cut_file.toLocalFile());
                
                if (info.fileName().compare(cut_file_info.fileName()) == 0){
                    QIcon icon = m_iconProvider.icon(info);
                    QPixmap pix = icon.pixmap(m_iconSize, QIcon::Disabled, QIcon::On);
                    return QIcon(pix);
                }
            }
                
            /*QString ext = info.suffix();
            
            if (info.isDir())
                ext = "directory";
            else if (info.fileName().isEmpty())
                ext = info.fileName();*/

            return m_iconProvider.icon(info);
                
            break;
        
        case Qt::ToolTipRole:
            return info.fileName();
            
        default:
            break;
    }
    
    //return QVariant();
     
    return QFileSystemModel::data(index, role);
}

void ScopeModel::setCutFilesList(QList<QUrl> urlList)
{
    cut_files.append(urlList);
}

void ScopeModel::clearCutFiles()
{
    cut_files.clear();
}

void ScopeModel::setIconSize(int width, int height)
{
    m_iconSize.scale(width, height, Qt::IgnoreAspectRatio);
    m_iconProvider.setIconSize(width, height);
}

void ScopeModel::showThumbnails(bool show)
{
    m_iconProvider.showThumbnails(show);
}

QIcon ScopeModel::icon(const QFileInfo& fileInfo)
{
    return m_iconProvider.icon(fileInfo);
}
