/* 
 *  Copyright (c) 2009, 2010 Mariano Street <mctpyt@delphosproject.org>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "systemdir.hpp"
#include "defaults.hpp"

#include <QtCore/QByteArray>
#include <QtCore/QFile>
#include <QtCore/QHash>
#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QStringList>


QString SystemDir::getDescription(const QString & p_path)
{
    QFile descriptionsFile(SYSTEMDIRDESCSFILE);
    
    if (! descriptionsFile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        // The descriptions file could not be opened.
        return "";
    }
    
    QByteArray        line;
    QList<QByteArray> lineParts;
    while (! descriptionsFile.atEnd())
    {
        line = descriptionsFile.readLine();
        line = line.trimmed();
        if (line.isEmpty())
        {
            continue;
        }
        // Line format:
        // /path/to/system/dir/ :Description.
        lineParts = line.split(':');
        if (lineParts[0].trimmed() == p_path)
        {
            descriptionsFile.close();
            // The path was found in the descriptions file, so it is a system
            // file.  The description is returned.
            return lineParts[1];
        }
    }
    
    descriptionsFile.close();
    // The path was not found in the descriptions file, so it is not a system
    // file..
    return "";
}


QHash<QString, QString> SystemDir::getDescriptions(const QStringList & p_pathList,
                                                  const QString &      p_basePath)
{
    QHash<QString, QString> result;
    QFile                   descriptionsFile(SYSTEMDIRDESCSFILE);
    
    if (! descriptionsFile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        // The descriptions file could not be opened.
        return QHash<QString,QString>();
    }
    
    QByteArray        line;
    QList<QByteArray> lineParts;
    while (! descriptionsFile.atEnd())
    {
        line = descriptionsFile.readLine();
        line = line.trimmed();
        if (line.isEmpty())
        {
            continue;
        }
        // Line format:
        // /path/to/system/dir/ :Description.
        lineParts = line.split(':');
        foreach (QString path, p_pathList)
        {
            if (lineParts[0].trimmed() == p_basePath + '/' + path)
            {
                // The given file list contains the system file path that is
                // currently being parsed, so add it to the result hash.
                result[path] = lineParts[1];
            }
        }
    }
    descriptionsFile.close();
    
    // Return a hash which contains those file paths from the given list that
    // also appear in the descriptions file, so they are system files.
    return result;
}
