/***************************************************************************
 *   Copyright (C) 2011 by Carlos Pais <freemind@lavabit.com>              *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QFile>
#include <QTextStream>
#include <QtDebug>
#include <stdlib.h>
#include <QProcess>
#include "utils.h"


QStringList getData(const QString& path)
{
    QStringList fileContents;
    QFile file( path );
    
    if ( path.isEmpty() ) {
        //qWarning() << Q_FUNC_INFO << "Empty file path given, doing nothing";
        return fileContents;
    }
    else if ( ! file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
        //qWarning() << Q_FUNC_INFO << "Could not read from file" << path;
        return fileContents;
    }
    
    QTextStream in( &file );
    while ( ! in.atEnd() ) 
        fileContents.push_back(in.readLine().trimmed());

    return fileContents;
}

QString getValueFromData(const QStringList& data, const QString& key)
{
    QString value("");
    
    foreach(QString line, data)
    {
        if ( line.contains(key, Qt::CaseInsensitive) && line.contains("=") )
        {
           value = line.split("=")[1];
           break;
        }
    }
    
    return value;
} 

QString getValueFromLine(const QString& line)
{
    if ( line.contains("=") )
        return line.split("=")[1];
        
    return "";
}

void removeStringDuplicates(QStringList& data)
{
    int size = data.size();
    QString str;
    int i,j;
    
    for(i=0; i < size; i++)
    {
        if ( i >= data.size() )
            break;
        
        str = data[i];
        for(j=0; j < data.size(); j++)
        {
            if ( i == j ) continue;
            
            if ( str == data[j] )
            {
                data.removeAt(j);
            }
        }   
    }
}

QString getCommandOutput(const QString & cmd)
{
    QProcess proc;
    proc.start(cmd);
	proc.waitForStarted();
	proc.waitForFinished();
    return proc.readAll().trimmed();
}

QString getDesktopEnv()
{
    QString output;
    
    if (  QString("true") == getenv("KDE_FULL_SESSION") )
        return "kde";
    else if ( ! getCommandOutput("pidof gnome-session").isEmpty() )
        return "gnome";
    else if ( getCommandOutput("xprop -root _DT_SAVE_MODE").contains("xfce") )
        return "xfce";
    else if ( ! getCommandOutput("pidof lxsession").isEmpty() )
        return "lxde";
        
    return "";
}
